# -*- coding: utf-8 -*-
#
# Created on Oct 15, 2013
# @author: Jiri Spilka
# http://bio.felk.cvut.cz/~spilkaj
# 2013, CTU in Prague, FEE, Dept. of Cybernetics
#
# Licensed under the terms of the GNU GENERAL PUBLIC LICENSE
# (see ctgViewerLite.py for details)

import sys
from PyQt4.QtGui import QApplication, QWidget, QFont


from ClinInfoWidgetGui import Ui_ClinInfoWidget


# list_clin_info = ['pH','BDecf','pCO2','BE','Apgar1','Apgar5','NICU_days','Seizures','HIE',
#                   'Intubation','Main_diag','Other_diag','Gest_weeks','Weight_g','Sex',
#                   'Age','Gravidity','Parity','Diabetes','Hypertension','Preeclampsia',
#                   'Liq_praecox','Pyrexia','Meconium','Presentation','Induced','Istage',
#                   'NoProgress','CK_KP','IIstage','Deliv_type','dbID','Rec_type','Pos_IIst','Sig2Birth']

dictClinInfoFullNames = dict()
dictClinInfoFullNames['pH'] = 'pH'
dictClinInfoFullNames['BDecf'] = 'BDecf'
dictClinInfoFullNames['BE'] = 'BE'
dictClinInfoFullNames['Apgar1'] = 'Apgar 1 min.'
dictClinInfoFullNames['Apgar5'] = 'Apgar 5 min.'
dictClinInfoFullNames['NICU_days'] = 'NICU [days]'
dictClinInfoFullNames['Seizures'] = 'Seizures'
dictClinInfoFullNames['HIE'] = 'HIE'
dictClinInfoFullNames['Intubation'] = 'Intubation'
dictClinInfoFullNames['Main_diag'] = 'Main diagnosis'
dictClinInfoFullNames['Other_diag'] = 'Other diagnoses'
dictClinInfoFullNames['Gest_weeks'] = 'Gestation [weeks]'
dictClinInfoFullNames['Weight_g'] = 'weight [g]'
dictClinInfoFullNames['Sex'] = 'Sex'
dictClinInfoFullNames['Age'] = 'Age'
dictClinInfoFullNames['Gravidity'] = 'Gravidity'
dictClinInfoFullNames['Parity'] = 'Parity'
dictClinInfoFullNames['Diabetes'] = 'Diabetes'
dictClinInfoFullNames['Hypertension'] = 'Hypertension'
dictClinInfoFullNames['Preeclampsia'] = 'Preeclampsia'
dictClinInfoFullNames['Liq_praecox'] = 'Liq_praecox'
dictClinInfoFullNames['Pyrexia'] = 'Pyrexia'
dictClinInfoFullNames['Meconium'] = 'Meconium'
dictClinInfoFullNames['Presentation'] = 'Presentation'
dictClinInfoFullNames['Induced'] = 'Induced'
dictClinInfoFullNames['Istage'] = 'Istage [min.]'
dictClinInfoFullNames['IIstage'] = 'IIstage [min.]'
dictClinInfoFullNames['NoProgress'] = 'NoProgress'
dictClinInfoFullNames['CK_KP'] = 'CK_KP'
dictClinInfoFullNames['Deliv_type'] = 'Delivery type'
dictClinInfoFullNames['Rec_type'] = 'Record type'
dictClinInfoFullNames['Pos_IIst'] = 'Position II stage [samples]'
dictClinInfoFullNames['Sig2Birth'] = 'Distance Sig2Birth [min.]'


class ClinInfo(QWidget):
    '''
    classdocs
    '''


    def __init__(self,parent = None):
        '''
        Constructor
        '''
        QWidget.__init__(self, parent)

        self.ui = Ui_ClinInfoWidget()
        self.ui.setupUi(self)

        self._textClear = 'N/A'

        self._fontChecked = QFont()
        self._fontChecked.setStrikeOut(False)

        self._fontNotChecked = QFont()
        self._fontNotChecked.setStrikeOut(True)

        self.clearAll()

    def clearAll(self):
        self.ui.label_apgar1.setText(self._textClear)
        self.ui.label_apgar5.setText(self._textClear)
        self.ui.label_bdecf.setText(self._textClear)
        self.ui.label_be.setText(self._textClear)
        self.ui.label_gravidity.setText(self._textClear)
        self.ui.label_IIstage.setText(self._textClear)
        self.ui.label_Istage.setText(self._textClear)
        self.ui.label_maternal_age.setText(self._textClear)
        self.ui.label_parity.setText(self._textClear)
        self.ui.label_pco2.setText(self._textClear)
        self.ui.label_ph.setText(self._textClear)
        self.ui.label_presentation.setText(self._textClear)
        self.ui.label_sex.setText(self._textClear)
        self.ui.label_sig2birth.setText(self._textClear)
        self.ui.label_week_gestation.setText(self._textClear)
        self.ui.label_weigth.setText(self._textClear)

        # risk factors
        self.__setLabelNotPresent(self.ui.label_diabetes)
        self.__setLabelNotPresent(self.ui.label_fever)
        self.__setLabelNotPresent(self.ui.label_hypertension)
        self.__setLabelNotPresent(self.ui.label_libpreacox)
        self.__setLabelNotPresent(self.ui.label_meconium)
        self.__setLabelNotPresent(self.ui.label_preeclampsia)

    def setAll(self, dictClinInfo):

        self.clearAll()

        #print dictClinInfo.keys()
        self.__setLabel(dictClinInfo, 'Apgar1', self.ui.label_apgar1)
        self.__setLabel(dictClinInfo, 'Apgar5', self.ui.label_apgar5)
        self.__setLabel(dictClinInfo, 'BDecf', self.ui.label_bdecf)
        self.__setLabel(dictClinInfo, 'BE', self.ui.label_be)
        self.__setLabel(dictClinInfo, 'Gravidity', self.ui.label_gravidity)
        self.__setLabel(dictClinInfo, 'IIstage', self.ui.label_IIstage)
        self.__setLabel(dictClinInfo, 'Istage', self.ui.label_Istage)
        self.__setLabel(dictClinInfo, 'Age', self.ui.label_maternal_age)
        self.__setLabel(dictClinInfo, 'Parity', self.ui.label_parity)
        self.__setLabel(dictClinInfo, 'pCO2', self.ui.label_pco2)
        self.__setLabel(dictClinInfo, 'pH', self.ui.label_ph)
        self.__setLabel(dictClinInfo, 'Presentation', self.ui.label_presentation)
        self.__setLabel(dictClinInfo, 'Sex', self.ui.label_sex)
        self.__setLabel(dictClinInfo, 'Sig2Birth', self.ui.label_sig2birth)
        self.__setLabel(dictClinInfo, 'Gest_weeks', self.ui.label_week_gestation)
        self.__setLabel(dictClinInfo, 'Weight_g', self.ui.label_weigth)

        # ['Seizures', '', 'NoProgress', 'Intubation', '', 'CK_KP', '', '',
        # 'Induced', '', '', '', 'Pos_IIst', 'Deliv_type', '',
        #'', 'HIE', '', '', '', 'NICU_days', 'Main_diag', 'Pyrexia',
        #'', '', 'Rec_type', '', '', '', 'dbID', '', '', '', 'Other_diag', '']


        self.__setRiskFactors(dictClinInfo, 'Diabetes', self.ui.label_diabetes)
        self.__setRiskFactors(dictClinInfo, '', self.ui.label_fever)
        self.__setRiskFactors(dictClinInfo, 'Hypertension', self.ui.label_hypertension)
        self.__setRiskFactors(dictClinInfo, 'Liq_praecox', self.ui.label_libpreacox)
        self.__setRiskFactors(dictClinInfo, 'Meconium', self.ui.label_meconium)
        self.__setRiskFactors(dictClinInfo, 'Preeclampsia', self.ui.label_preeclampsia)


    def __setLabelNotPresent(self, label):
        label.setFont(self._fontNotChecked)
        label.setEnabled(False)

    def __setLabel(self, dictCI, key, label):
        if key in dictCI:
            label.setText(str(dictCI[key]))

    def __setRiskFactors(self, dictCI, key, label):
        if key in dictCI:
            if dictCI[key] > 0:
                label.setFont(self._fontChecked)
                label.setEnabled(True)
                #checkBox.setCheckable(False)


def main():
    app = QApplication(sys.argv)

    window = ClinInfo()
    window.show()
    sys.exit(app.exec_())

if __name__ == '__main__':
    main()
