# -*- coding: utf-8 -*-
#
# Created on Oct 15, 2013
# @author: Jiri Spilka
# http://bio.felk.cvut.cz/~spilkaj
# 2013, CTU in Prague, FEE, Dept. of Cybernetics
#
# Licensed under the terms of the GNU GENERAL PUBLIC LICENSE
# (see ctgViewerLite.py for details)

from os.path import expanduser, join
from datetime import datetime
import os
import shutil
import logging

import Common
import ConfigParser

class EnumIniVar:
    """
    variables in ini file
    """
    lastUsedDirFiles = 'lastUsedDirFiles'
    dockClinInformationVisible = 'dockClinInformationVisible'
    dataBrowserSelectedAttributes = 'dataBrowserSelectedAttributes'

class ConfigIni():

    def __init__(self):

        self.__ini_var_section = dict()
        self.__ini_var_section[EnumIniVar.lastUsedDirFiles] = 'Directories'
        self.__ini_var_section[EnumIniVar.dockClinInformationVisible] = 'GUI'
        self.__ini_var_section[EnumIniVar.dataBrowserSelectedAttributes] = 'GUI'

        self.__ini_defaults = dict()
        self.__ini_defaults[EnumIniVar.lastUsedDirFiles] = ' '
        self.__ini_defaults[EnumIniVar.dockClinInformationVisible] = True
        self.__ini_defaults[EnumIniVar.dataBrowserSelectedAttributes] = 'pH, BDecf'

        self._ini_var = dict()

        self._configParser = ConfigParser.ConfigParser()

        self._log = logging.getLogger(ConfigStatic.logger_name)

        #self.dirLastUsed = ''
        #self.dockClinInformationVisible = 0
        #self.dataBrowserSelectedAttributes = ''

        #self._lSections = ['Directories','GUI']

        #self.__ensure_settings_ini()
        self.read_config()

    def __ensure_settings_ini(self):
        default_ini = 'default.ini'
        app_path = Common.get_application_path()
        file2copy = os.path.join(app_path,default_ini)
        if not Common.file_exists(ConfigStatic.setting_file):
            shutil.copy(file2copy, ConfigStatic.setting_file)
            self._log.info("ConfigStatic file not exists, copy from application path")

    def read_config(self, ini_file = None):

        if ini_file is None:
            ini_file = ConfigStatic.setting_file

        self._log.info("reading config file {0}".format(ini_file))
        #if not Common.file_exists(ini_file):
        #    return

        self._configParser.read(ConfigStatic.setting_file)

        for key in self.__ini_var_section:

            if key == EnumIniVar.dockClinInformationVisible:
                p_get = self._configParser.getboolean
            else:
                p_get = self._configParser.get

            try:
                self._ini_var[key] = p_get(self.__ini_var_section[key], key)
            except:
                self._ini_var[key] = self.__ini_defaults[key]

        self.attributesRemoveApostrophsAndWhiteSpaces()

        #try:
        #    self.dirLastUsed = self._configParser.get('DEFAULT', 'lastUsedDirFiles')
        #    #self.dirLastUsed = self._configParser.get('Directories', 'lastUsedDirFiles')
        #    self.dockClinInformationVisible = self._configParser.getboolean('GUI', 'dockClinInformationVisible')
        #    self.dataBrowserSelectedAttributes = self._configParser.get('GUI', 'dataBrowserSelectedAttributes')
        #    self.attributesRemoveApostrophsAndWhiteSpaces()
        #except Exception, e:
        #    pass
        #    #raise Exception('Error when reading config file {0}, message:{1}'.format(ini_file,e))

    def attributesRemoveApostrophsAndWhiteSpaces(self):

        temp = self.get_var(EnumIniVar.dataBrowserSelectedAttributes)
        temp = temp.split(',')

        # remove apostrophes and whitespaces
        cnt = 0
        for dummy in temp:
            temp[cnt] = (str(temp[cnt].replace("'",""))).strip()
            cnt += 1

        self.set_var(EnumIniVar.dataBrowserSelectedAttributes, temp)

    def write_config(self):
        # lets create that config file for next time...
        cfgf = open(ConfigStatic.setting_file, 'w+')

        # add the settings to the structure of the file, and lets write it out...
        #for l in self._lSections:
        #    try:
        #        self._configParser.add_section(l)
        #    except Exception,msg:
        #        self._log.info("A section exist {0}".format(msg))
        for key in self.__ini_var_section:
            try:
                self._configParser.add_section(self.__ini_var_section[key])
            except Exception,msg:
                self._log.info("A section exist {0}".format(msg))

            temp = self.get_var(key)

            if key == EnumIniVar.dataBrowserSelectedAttributes:
                temp = ','.join(temp)

            self._configParser.set(self.__ini_var_section[key], key, temp)

        #self._configParser.set('GUI', 'dataBrowserSelectedAttributes', ','.join(self.dataBrowserSelectedAttributes))
        #self._configParser.set('Directories', 'lastUsedDirFiles', self.dirLastUsed)

        self._configParser.write(cfgf)
        cfgf.close()

        self._log.info("Ini file has been written successfully")

    def get_var(self,name):
        try:
            value = self._ini_var[name]
        except Exception, e:
            raise Exception('Error - value {0} not in settings ini, message:{1}'.format(name,e))

        return value

    def set_var(self,name,value):
        try:
            self._ini_var[name] = value
        except Exception, e:
            raise Exception('Error - value {0} not in settings ini, message:{1}'.format(name,e))

class ConfigStatic:

    _configParser = ConfigParser.ConfigParser()

    user_directory = expanduser("~/.ctgViewerLite")

    # logging
    logging_level = logging.ERROR
    logger_name = 'ctgViewerLite'
    log_path = join(user_directory,'logs')
    log_name = 'log_' + datetime.today().strftime("%Y-%m-%d_%H-%M-%S") + '.log'
    log_file = join(log_path, log_name)

    #temp directory
    temp_data = join(user_directory,'temp_data')

    # date time format
    date_format = '%d.%m.%Y'
    time_format = '%H:%M:%S:%f'

    setting_file = join(user_directory,'settings.ini')