# -*- coding: utf-8 -*-
#
# Created on Mar 8, 2014
# @author: Jiri Spilka
# http://bio.felk.cvut.cz/~spilkaj
# 2013, CTU in Prague, FEE, Dept. of Cybernetics
#
# Licensed under the terms of the GNU GENERAL PUBLIC LICENSE
# (see ctgViewerLite.py for details)

"""
DataBrowserSelectAttributesForm
-------------------------------

Select attributes that will be used for browsing database files

"""

import sys
import logging

from PyQt4 import QtGui, QtCore, Qt
from DataBrowserSelectAttributesGui import Ui_DataBrowserSelectAttributesWin
from Config import ConfigStatic, ConfigIni
from ClinInfo import dictClinInfoFullNames

class DataBrowserSelectAttributesForm(QtGui.QWidget):

    #userSettingsChanged = pyqtSignal()

    def __init__(self,configIni = None):
        QtGui.QWidget.__init__(self)

        self.ui = Ui_DataBrowserSelectAttributesWin()
        self.ui.setupUi(self)

        self._log = logging.getLogger(ConfigStatic.logger_name)
        self._log.info('passed')

        # CONFIG
        if configIni is None:
            self._configIni = ConfigIni()

        self._modelSelectedAtt = QtGui.QStandardItemModel()
        self._modelAvailableAtt = QtGui.QStandardItemModel()

        self.ui.listSelected.setSelectionMode(Qt.QAbstractItemView.ExtendedSelection)
        self.ui.listAvailable.setSelectionMode(Qt.QAbstractItemView.ExtendedSelection)

        self.connect(self.ui.buttonBox.button(QtGui.QDialogButtonBox.Save), QtCore.SIGNAL('clicked()'),self._save)
        self.connect(self.ui.buttonBox.button(QtGui.QDialogButtonBox.Close), QtCore.SIGNAL('clicked()'),self._cancel)
        self.ui.buttonBox.button(QtGui.QDialogButtonBox.Save).setEnabled(False)

        self.connect(self.ui.btnRigthAll, QtCore.SIGNAL('clicked()'),self._btnRigthAll)
        self.connect(self.ui.btnLeftAll, QtCore.SIGNAL('clicked()'),self._btnLeftAll)
        self.connect(self.ui.btnlLeftSelected, QtCore.SIGNAL('clicked()'),self._btnLeftSelected)
        self.connect(self.ui.btnRigthSelected, QtCore.SIGNAL('clicked()'),self._btnRigthSelected)

        self._selectedAtt = self._configIni.dataBrowserSelectedAttributes

        self._availableAtt = dictClinInfoFullNames
        self.ui.listSelected.setModel(self._modelSelectedAtt)

        self.__setModelListViews()

    def __setModelListViews(self):

        for att in self._availableAtt.keys():
            #print att

            if att in self._selectedAtt:
                self._modelSelectedAtt.appendRow(QtGui.QStandardItem(self._availableAtt[att]))
            else:
                self._modelAvailableAtt.appendRow(QtGui.QStandardItem(self._availableAtt[att]))

        self.ui.listSelected.setModel(self._modelSelectedAtt)
        self.ui.listAvailable.setModel(self._modelAvailableAtt)

        self._modelSelectedAtt.sort(0)
        self._modelAvailableAtt.sort(0)

    def _btnLeftAll(self):

        self.ui.listAvailable.selectAll()
        self._btnLeftSelected()

    def _btnLeftSelected(self):
        """
        move the selected indicies to the left column
        """
        indicies = self.ui.listAvailable.selectedIndexes()

        if len(indicies) > 0:
            row = indicies[0].row()
            item = QtGui.QStandardItem(self._modelAvailableAtt.item(row))
            self._modelAvailableAtt.removeRow(row)
            self._modelSelectedAtt.appendRow(item)
            self._btnLeftSelected() # recursively remove
            self.ui.buttonBox.button(QtGui.QDialogButtonBox.Save).setEnabled(True)


    def _btnRigthAll(self):

        self.ui.listSelected.selectAll()
        self._btnRigthSelected()

    def _btnRigthSelected(self):

        indicies = self.ui.listSelected.selectedIndexes()

        if len(indicies) > 0:
            row = indicies[0].row()
            item = QtGui.QStandardItem(self._modelSelectedAtt.item(row))
            self._modelSelectedAtt.removeRow(row)
            self._modelAvailableAtt.appendRow(item)
            self._btnRigthSelected() # recursively remove
            self.ui.buttonBox.button(QtGui.QDialogButtonBox.Save).setEnabled(True)

    def _save(self):

        self._selectedAtt = list() # delete list of selected attributes

        # get items in model
        for i in range(self._modelSelectedAtt.rowCount()):
            #print i
            item = self._modelSelectedAtt.item(i)
            s = item.text()

            dict1 = self._availableAtt.iteritems()
            # iterate through all attributes to get key under which the attributes are stored in physionet header
            for d in dict1:
                if s == d[1]:
                    self._selectedAtt.append(d[0])


        self._configIni.dataBrowserSelectedAttributes = str(self._selectedAtt).strip('[]')
        self._configIni.write_config()
        self.ui.buttonBox.button(QtGui.QDialogButtonBox.Save).setEnabled(False)

    def _cancel(self):
        self.close()

    def closeEvent(self, event):

        if self.ui.buttonBox.button(QtGui.QDialogButtonBox.Save).isEnabled() == False:
            return

        msgBoxWarning = QtGui.QMessageBox()
        msgBoxWarning.setText('There are unsaved changes.')
        msgBoxWarning.setInformativeText('Save them now?')
        msgBoxWarning.setIcon(QtGui.QMessageBox.Warning)
        msgBoxWarning.setStandardButtons(QtGui.QMessageBox.Yes | QtGui.QMessageBox.No);
        ret = msgBoxWarning.exec_()

        if ret == QtGui.QMessageBox.Yes:
            self._save()

def main():
    app = QtGui.QApplication(sys.argv)
    window = DataBrowserSelectAttributesForm()
    window.show()
    sys.exit(app.exec_())

if __name__ == '__main__':
        main()