# -*- coding: utf-8 -*-
#
# Created on Oct 15, 2013
# @author: Jiri Spilka
# http://bio.felk.cvut.cz/~spilkaj
# 2013, CTU in Prague, FEE, Dept. of Cybernetics
#
# Licensed under the terms of the GNU GENERAL PUBLIC LICENSE
# (see ctgViewerLite.py for details)


from PyQt4 import QtGui
from PyQt4.Qt import QCoreApplication, QMessageBox
import sys
import os
import urllib
import io
import hashlib

# local imports
from AboutGui import Ui_DialogAbout
from ConvertFileUI import Ui_ConvertFile
from DownloadCtuUhbUI import Ui_DownloadCtuUhb
from LoadWriteData import LoadData
import ClinInfo
import Common

class AboutDialog(QtGui.QDialog):

    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self)
        self.ui = Ui_DialogAbout()
        self.ui.setupUi(self)

        pal = QtGui.QPalette()
        pal.setColor(QtGui.QPalette.Base, pal.background().color())
        self.ui.textBrowser.setPalette(pal);

        self.ui.textBrowser.setOpenExternalLinks(True)

        #self.ui.label.setPixmap(QtGui.QPixmap('images/BioDat_logo_final_small.jpg'))

        #self.ui.textEdit.setReadOnly(False)
        #print self.ui.textEdit.textColor()
        #self.ui.textEdit.setTextBackgroundColor(a)
        #self.ui.textEdit.setTextColor(a)
        #print self.ui.textEdit.textColor()

        #print self.ui.textEdit.isReadOnly()


class ConvertFileForm(QtGui.QWidget):

    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self)

        self.ui = Ui_ConvertFile()
        self.ui.setupUi(self)

        pal = QtGui.QPalette()
        pal.setColor(QtGui.QPalette.Base, pal.background().color())
        self.ui.textEditInfo.setPalette(pal);
        self.ui.textEditInfo.setReadOnly(True)

        self.listAllowedFormats = ['dat','csv','mat', 'txt']
        self.listFormatTooltips = ['physionet format','comma separated value','matlab format','BDI txt']

        self._filesToConvert = []
        self._dataLoader = LoadData()

        self._dictClinInfo = ClinInfo.dictClinInfoFullNames

        self._setupComboBoxes()

        self.ui.btnBrowseSource.clicked.connect(self._openDirectorySource)
        self.ui.btnBrowseDest.clicked.connect(self._openDirectoryDest)
        self.ui.btnConvert.clicked.connect(self._convert)
        self.ui.btnStop.clicked.connect(self._stop)

        self.msgBoxError = QtGui.QMessageBox()
        self.msgBoxError.setDefaultButton(QtGui.QMessageBox.Close)
        self.msgBoxError.setIcon(QtGui.QMessageBox.Critical)

        self._bAbort = False
        self.ui.btnStop.setEnabled(False)

        self.ui.lnSource.setText('/home/jirka/data/igabrno/CTU_UHB_physionet')
        self.ui.lnDestination.setText('/home/jirka/data/igabrno/CTU_UHB_physionet')

        #self.ui.lnSource.editingFinished.connect(self.checkSourceDirForFiles)

    def _setupComboBoxes(self):

        cnt = 0
        self.ui.cbFrom.addItem('dat')
        self.ui.cbTo.addItem('csv')

        #for l in self.listAllowedFormats:
        #    self.ui.cbFrom.addItem(l)
        #    self.ui.cbTo.addItem(l)
        #    self.ui.cbFrom.setItemData(cnt, self.listFormatTooltips[cnt], Qt.Qt.ToolTipRole)
        #    self.ui.cbTo.setItemData(cnt, self.listFormatTooltips[cnt], Qt.Qt.ToolTipRole)
        #    cnt += 1

        self.ui.cbFrom.setCurrentIndex(0)
        self.ui.cbTo.setCurrentIndex(0)

    def _openDirectorySource(self):
        open_dialog = QtGui.QFileDialog(self, 'Open directory', '')
        dir1 = open_dialog.getExistingDirectory()

        if dir == "":
            return -1
        else:
            dir1 = str(dir1)
            self.ui.lnSource.setText(dir1)
            self.checkSourceDirForFiles()

        return 0

    def _openDirectoryDest(self):
        open_dialog = QtGui.QFileDialog(self, 'Open directory', '')
        dir1 = open_dialog.getExistingDirectory()

        if dir == "":
            return -1
        else:
            dir1 = str(dir1)
            self.ui.lnDestination.setText(dir1)

        return 0

    def checkDirectoryExists(self,dir1):

        if not os.path.exists(dir1):
            raise Exception('Chosen directory do not exists: {0}'.format(dir1))

    def checkSourceDirForFiles(self):

        dir1 = str(self.ui.lnSource.text())
        ext_mask = str(self.ui.cbFrom.itemText(self.ui.cbFrom.currentIndex()))

        self.checkDirectoryExists(dir1)

        files = Common.get_files_with_ext_mask(dir1,ext_mask)
        files.sort()

        self.ui.labFile2Convert.setText(str(len(files)))
        self._filesToConvert = files

    def _stop(self):
        self.ui.btnStop.setEnabled(False)
        self._bAbort = True

    def _convert(self):

        self.ui.textEditInfo.clear()
        self.ui.btnConvert.setEnabled(False)
        self.ui.btnStop.setEnabled(True)

        # check source files
        extFrom = str(self.ui.cbFrom.currentText())
        extTo = str(self.ui.cbTo.currentText())

        if extFrom == extTo:
            self.msgBoxError.setText('Cannot convert to the same format')
            self.msgBoxError.exec_()
            return

        dir1 = str(self.ui.lnSource.text())
        dir2 = str(self.ui.lnDestination.text())

        try:
            self.checkDirectoryExists(dir1)
        except Exception as ex:
            self.msgBoxError.setText('Error in source directory')
            self.msgBoxError.setInformativeText(ex.message)
            self.msgBoxError.exec_()
            return

        try:
            self.checkDirectoryExists(dir2)
        except Exception as ex:
            self.msgBoxError.setText('Error in destination directory')
            self.msgBoxError.setInformativeText(ex.message)
            self.msgBoxError.exec_()
            return

        self.checkSourceDirForFiles()

        if self._filesToConvert == 0:
            self.msgBoxError.setText('No files with format {0} in specified directory {1}',extFrom,dir1)
            self.msgBoxError.exec_()
            return

        if extTo == 'csv':
            sFileClinicalInfo = os.path.join(dir2,('clinical_information.csv'))
            fClinInfo = open(sFileClinicalInfo,'w+')

            cnt = 0
            for s in self._dictClinInfo.keys():
                fClinInfo.write(self._dictClinInfo[s])
                cnt += 1
                if cnt == len(self._dictClinInfo):
                    fClinInfo.write('\n')
                else:
                    fClinInfo.write(',')

        # run conversion
        for f in self._filesToConvert:

            if self._bAbort:
                self.ui.textEditInfo.append('ABORTED')
                return

            # convert FROM
            if extFrom == 'dat':
                aDataAll, lHeader = self._dataLoader.readPhysionetSignal16(f)

            # convert TO
            if extTo == 'dat':
                self.msgBoxError.setText('Not supported yet')
                self.msgBoxError.exec_()

            if extTo == 'csv':
                # file name
                try:
                    file_name = Common.get_filename_without_ext(f)
                    name_new = os.path.join(dir2,(file_name+'.'+extTo))

                    # data fhr, uc
                    self._dataLoader.writeCsvFile(name_new, aDataAll)

                    # clinical information
                    cnt = 0
                    dictHeaderInfo = lHeader[2]
                    for l in self._dictClinInfo.keys():
                        if l in dictHeaderInfo:
                            fClinInfo.write(str(dictHeaderInfo[l]))
                        else:
                            fClinInfo.write('')

                        cnt += 1
                        if cnt == len(self._dictClinInfo):
                            fClinInfo.write('\n')
                        else:
                            fClinInfo.write(',')

                    self.ui.textEditInfo.append('File {0} - Converted'.format(file_name+'.'+extTo))
                    QCoreApplication.processEvents()
                except Exception,msg:
                    self.ui.textEditInfo.append('File {0} - ERROR: {1}'.format(file_name+'.'+extTo,msg))
                    QCoreApplication.processEvents()

        if extTo == 'csv':
            self.ui.textEditInfo.append('All files converted successfully')
            self.ui.textEditInfo.append('Clinical information was saved into file: {0}'.format(sFileClinicalInfo))
            fClinInfo.close()

class DownoladDbForm(QtGui.QWidget):

    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self)

        self.ui = Ui_DownloadCtuUhb()
        self.ui.setupUi(self)

        pal = QtGui.QPalette()
        pal.setColor(QtGui.QPalette.Base, pal.background().color())
        self.ui.textEditInfo.setPalette(pal);
        self.ui.textEditInfo.setReadOnly(True)

        self.ui.btnBrowseDirs.clicked.connect(self._openDirectorySource)
        self.ui.btnStart.clicked.connect(self._startDownload)
        self.ui.btnStop.clicked.connect(self._stop)

        self.msgBoxError = QtGui.QMessageBox()
        self.msgBoxError.setDefaultButton(QtGui.QMessageBox.Close)
        self.msgBoxError.setIcon(QtGui.QMessageBox.Critical)

        self.msgBoxWarn = QtGui.QMessageBox()
        self.msgBoxWarn.setStandardButtons(QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
        self.msgBoxWarn.setIcon(QtGui.QMessageBox.Information)

        self._bAbort = False
        self.ui.btnStop.setEnabled(False)
        self.ui.progressBar.reset()

        self._MD5SUMS = 'MD5SUMS'
        self._attempsToDownloadFile = 5

    def _openDirectorySource(self):
        open_dialog = QtGui.QFileDialog(self, 'Open directory', '')
        dir1 = open_dialog.getExistingDirectory()

        if dir == "":
            return -1
        else:
            dir1 = str(dir1)
            self.ui.lnDestDir.setText(dir1)

        return 0

    def checkDirectoryExists(self,dir1):

        if not os.path.exists(dir1):
            raise Exception('Chosen directory do not exists: {0}'.format(dir1))

    def _startDownload(self):
        """
        prepare for downloading - chack the directories and url
        """

        self._bAbort = False

        #self.ui.lnDestDir.setText('/home/jirka/svn_working_copy/iga_brno/python/ctgViewerLite/src/testy/aaa')
        self._url = str(self.ui.lineEditUrl.text())
        self.ui.textEditInfo.clear()

        self._dirDest = str(self.ui.lnDestDir.text())

        try:
            self.checkDirectoryExists(self._dirDest)
        except Exception as ex:
            self.msgBoxError.setText('Error in destination directory')
            self.msgBoxError.setInformativeText(ex.message)
            self.msgBoxError.exec_()
            return

        files = Common.directory_listing(self._dirDest)

        ret = 0
        if len(files) > 0:
            self.msgBoxWarn.setText('The destination directory is not empty.')
            self.msgBoxWarn.setInformativeText('It is recommended to download database into an empty directory. Continue download?')
            ret = self.msgBoxWarn.exec_()

        if ret == QMessageBox.No:
            return

        # check if MD5SUMS exist
        name_dest = os.path.join(self._dirDest,self._MD5SUMS)
        fname,dummy = urllib.urlretrieve(self._url + self._MD5SUMS,name_dest)

        bNotFound = False
        with io.open(fname,'rt') as fin:
            for s in fin.readlines():
                ind = s.find(fname + ' was not found on this server')
                if ind > -1:
                    bNotFound = True

        if bNotFound:
            self.msgBoxError.setText('Error in the database url')
            s = 'File: '  + self._MD5SUMS + ' was not found at the server.'
            s += 'Download can not continue without this file. Please check the database url and make sure that file MD5SUMS exists.'
            self.msgBoxError.setInformativeText(s)
            self.msgBoxError.exec_()
            return
        else:
            # if all ok -> run the actual download
            self.ui.btnStart.setEnabled(False)
            self.ui.btnStop.setEnabled(True)
            if self.run_download() == 0:
                self.ui.textEditInfo.append('Download completed')
                self.ui.btnStart.setEnabled(True)
                self.ui.btnStop.setEnabled(False)

    def run_download(self):
        """
        Download all files from given url and MD5SUMS file
        """

        # set the progress bar
        md5_file = os.path.join(self._dirDest,self._MD5SUMS)
        nNrFiles = Common.get_mumber_lines(md5_file)
        self.ui.progressBar.setRange(1,nNrFiles)
        self.ui.progressBar.update()

        # process all files in file: self._MD5SUMS

        with io.open(md5_file,'rt') as fin:

            processed_files = 0
            for line in fin.readlines():

                if self._bAbort:
                    self.ui.textEditInfo.append('ABORTED')
                    return 1

                md5,fname = line.split('  ')
                md5 = md5.strip()
                fname = fname.strip()

                # do not download this file
                if fname.find('HEADER.shtml') > -1:
                    processed_files += 1
                    continue

                cnt = 0
                bMD5_OK = False
                name_dest = os.path.join(self._dirDest,fname)

                # for a given number of attempts try to download a file
                while cnt < self._attempsToDownloadFile:
                    cnt += 1
                    fname,dummy = urllib.urlretrieve(self._url + fname,name_dest)
                    QCoreApplication.processEvents()
                    bMD5_OK = self.check_md5(md5, fname)

                    if bMD5_OK:
                        break

                if bMD5_OK:
                    self.ui.textEditInfo.append("Downloaded file: " + name_dest)
                else:
                    s = "Failed to download file: " + name_dest
                    s += ' MD5SUM do not match'
                    self.ui.textEditInfo.append(s)

                processed_files += 1
                self.ui.progressBar.setValue(processed_files)
                QCoreApplication.processEvents()

        return 0

    def check_md5(self,md5expected,filename):
        return md5expected == self.md5_for_file(filename)

    def md5_for_file(self,filename):
        """
        get md5sum for a given filename
        """

        md5 = hashlib.md5()
        file1 = open(filename,'rb')

        while True:
            data = file1.read(65536)
            if not data:
                break
            md5.update(data)

        file1.close()
        return md5.hexdigest()

    def _stop(self):
        self.ui.btnStop.setEnabled(False)
        self._bAbort = True
        self.ui.btnStart.setEnabled(True)


def main():
    app = QtGui.QApplication(sys.argv)
    #window = ConvertFileForm()
    window = DownoladDbForm()
    window.show()
    sys.exit(app.exec_())

if __name__ == '__main__':
        main()
