# -*- coding: utf-8 -*-
#
# Created on Oct 15, 2013
# @author: Jiri Spilka
# http://bio.felk.cvut.cz/~spilkaj
# 2013, CTU in Prague, FEE, Dept. of Cybernetics
#
# Licensed under the terms of the GNU GENERAL PUBLIC LICENSE
# (see ctgViewerLite.py for details)

import sys

from PyQt4.QtGui import QWidget, QVBoxLayout, QApplication
from PyQwtWidgetGui import FhrPlot
from PyQwtWidgetGui import TocoPlot
from PyQwtNavigationPlot import PyQwtNavigationPlot



class PlotWidget(QWidget):
    """
    Class intended only for debugging
        * parent
    """

    def __init__(self, parent = None):
        QWidget.__init__(self, parent)

        self.setMinimumSize(800, 600)
        #self.setMaximumSize(1200,800)

        #self._styleManager = styleManager()
        #self._styleManager.setStyleManager(self._styleManager)

        vbox = QVBoxLayout()

        controledPlots = list()
        self.fhrPlot = FhrPlot(self)
        self.tocoPlot = TocoPlot(self)
        #self.tocoPlot.setSizePolicy(QSizePolicy.Minimum,QSizePolicy.Minimum)

        controledPlots.append(self.fhrPlot)
        controledPlots.append(self.tocoPlot)
        self.navPlot = PyQwtNavigationPlot(self, controledPlots)

        #self.plotTools = PyQwtPlotTools(self, self.fhrPlot, self._styleManager)
        #self.plot.setMargin(5)

        vbox.addWidget(self.fhrPlot)
        vbox.addWidget(self.tocoPlot)
        #vbox.addLayout(self.plotTools.ToolbarHBoxLayout)
        vbox.addWidget(self.navPlot)
        self.setLayout(vbox)
        #self.setCentralWidget(self.fhrPlot)
        #self.setMinimumSize(90, 90)
        #self.setMinimumSize(1000, 100)

    def setSamplingFreqAllPlots(self,fs):
        self.fhrPlot.setSamplingFreq(fs)
        self.tocoPlot.setSamplingFreq(fs)
        self.navPlot.setSamplingFreq(fs)

    def getSamplingFreq(self):
        return self._samplingFreq

    def plot(self, time_samples, fhr, toco, timeString = None):

        self.clearAllPlots()
        self.fhrPlot.plot(time_samples, fhr, timeString)
        self.tocoPlot.plot(time_samples, toco, timeString)
        self.navPlot.plot(time_samples, fhr, timeString)

    def clearAllPlots(self):
        self.fhrPlot.clearPlot()
        self.tocoPlot.clearPlot()
        self.navPlot.clearPlot()

    def updatePlots(self):
        self.fhrPlot.updateAxis()
        self.tocoPlot.updateAxis()
        self.navPlot.setXAxis()

    def resizeEvent(self, ev):
        self.updatePlots()

def main():
    app = QApplication(sys.argv)
    window = PlotWidget()
    window.show()
    window.updatePlots()
    sys.exit(app.exec_())

if __name__ == '__main__':
        main()

