# -*- coding: utf-8 -*-
#
# Created on Oct 15, 2013
# @author: Jiri Spilka
# http://bio.felk.cvut.cz/~spilkaj
# 2013, CTU in Prague, FEE, Dept. of Cybernetics
#
# Licensed under the terms of the GNU GENERAL PUBLIC LICENSE
# (see ctgViewerLite.py for details)

from PyQt4.QtCore import pyqtSignal, QPoint
from PyQt4 import Qt
from PyQt4.Qwt5 import QwtPlotCanvas, QwtPlot

bDEBUG = False

class PyQwtCanvasPicker(Qt.QObject):

    pointClicked = pyqtSignal(['QPoint'])

    def __init__(self, plot):
        Qt.QObject.__init__(self, plot)
        self.__selectedCurve = None
        self.__selectedPoint = -1
        self.__plot = plot

        canvas = plot.canvas()
        canvas.installEventFilter(self)

        # We want the focus, but no focus rect.
        # The selected point will be highlighted instead.
        canvas.setFocusPolicy(Qt.Qt.StrongFocus)
        canvas.setCursor(Qt.Qt.PointingHandCursor)
        canvas.setFocusIndicator(QwtPlotCanvas.ItemFocusIndicator)
        canvas.setFocus()

    #def event(self, event):
    #    if event.type() == Qt.QEvent.User:
    #        return True
    #    #return Qt.QObject.event(event)
    #    return event

    def eventFilter(self, object, event):
        """
        if event.type() == Qt.QEvent.FocusIn:
            self.__showCursor(True)
        if event.type() == Qt.QEvent.FocusOut:
            self.__showCursor(False)
        """

        if event.type() == Qt.QEvent.Paint:
            Qt.QApplication.postEvent(self, Qt.QEvent(Qt.QEvent.User))

        elif (event.type() == Qt.QEvent.MouseButtonPress) | (event.type() == Qt.QEvent.MouseMove):


            clickedPoint = self._computePointClicked(event)
            self._pointClicked(clickedPoint)
            return True

        return QwtPlot.eventFilter(self, object, event)

    def _computePointClicked(self, event):
        """
        The clicked point stored in event.x() and event.y() is transformed to x and y axis for
        particular plot attached.

        :param event: event
        :type  event: QEvent
        :rtype: QPoint
        """
        clickedPointX = self.__plot.invTransform(QwtPlot.xBottom, event.pos().x())
        clickedPointY = self.__plot.invTransform(QwtPlot.xBottom, event.pos().y())

        if bDEBUG:
            print event.pos()
            print clickedPointX
            print clickedPointY

        return QPoint(int(clickedPointX), int(clickedPointY))

    def _pointClicked(self, point):
        """
        This signal is emitted when user click on a plot.
        """
        self.pointClicked.emit(point)
