# -*- coding: utf-8 -*-
#
# Created on Oct 15, 2013
# @author: Jiri Spilka
# http://bio.felk.cvut.cz/~spilkaj
# 2013, CTU in Prague, FEE, Dept. of Cybernetics
#
# Licensed under the terms of the GNU GENERAL PUBLIC LICENSE
# (see ctgViewerLite.py for details)

import unittest
import os

from LoadWriteData import EnumVariableName as Enum
from LoadWriteData import LoadData
import Init

class TestReadWrite(unittest.TestCase):

    def setUp(self):
        Init.init()
        self._dataLoader = LoadData()

    def tearDown(self):
        pass

    def test_read_header(self):

        file1 = os.path.join('unittest_files','1001.hea')
        lHeader,nNrSignals,nFs,nNrSamples = self._dataLoader.readPhysionetHeader(file1)

        dict1= lHeader[0]
        dict2= lHeader[1]

        self.assertEqual(nNrSignals, 2)
        self.assertEqual(nFs,4)
        self.assertEqual(nNrSamples, 19200)

        self.assertEqual(dict1['format'], 16)
        self.assertEqual(dict2['format'], 16)
        self.assertEqual(dict1['gain'], 100)
        self.assertEqual(dict2['gain'], 100)

        self.assertEqual(dict1['firstvalue'], 15050)
        self.assertEqual(dict2['firstvalue'], 700)

    def test_read_signal(self):

        file1 = os.path.join('unittest_files','1001.dat')
        aData, dummy = self._dataLoader.readPhysionetSignal16(file1)

        fhr = aData[Enum.fhr]
        uc = aData[Enum.uc]

        self.assertEqual(fhr[0], 150.5)
        self.assertEqual(fhr[3], 151.25)
        self.assertEqual(fhr[18978], 94.75)

        self.assertEqual(uc[0], 7.0)
        self.assertEqual(uc[1], 8.5)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()
