# -*- coding: utf-8 -*-
#
# Created on Oct 15, 2013
# @author: Jiri Spilka
# http://bio.felk.cvut.cz/~spilkaj
# 2013, CTU in Prague, FEE, Dept. of Cybernetics
#
# Licensed under the terms of the GNU GENERAL PUBLIC LICENSE
# (see ctgViewerLite.py for details)


import unittest
import os

from LoadWriteData import EnumVariableName as Enum
from LoadWriteData import LoadData
import numpy as np
import Init

class TestReadWrite(unittest.TestCase):

    def setUp(self):
        Init.init()
        self._dataLoader = LoadData()

    def tearDown(self):
        pass

    def test_read_csv(self):

        file1 = os.path.join('unittest_files','1001.csv')
        aData,dummy = self._dataLoader.readCsvFile(file1)

        timestamp = aData[Enum.timestamp]
        fhr = aData[Enum.fhr]
        uc = aData[Enum.uc]

        self.assertEqual(timestamp[0], 1)
        self.assertEqual(timestamp[1], 2)

        self.assertEqual(fhr[0], 150.5)
        self.assertEqual(fhr[3], 151.25)
        self.assertEqual(fhr[18978], 94.75)

        self.assertEqual(uc[0], 7.0)
        self.assertEqual(uc[1], 8.5)

    def test_write_read_csv(self):

        t1 = np.zeros((2,1),np.int)
        t2 = np.zeros((2,1),np.float64)
        t3 = np.zeros((2,1),np.float64)
        aDataAll = dict()

        t1[0,0] = 1
        t1[1,0] = 2
        t2[0,0] = 155.5
        t2[1,0] = 155
        t3[0,0] = 7.8
        t3[1,0] = 8

        aDataAll[Enum.timestamp] = t1
        aDataAll[Enum.fhr] = t2
        aDataAll[Enum.uc] = t3

        file1 = os.path.join('unittest_files','test_write_read.csv')
        self._dataLoader.writeCsvFile(file1, aDataAll)

        data_read,dummy = self._dataLoader.readCsvFile(file1)

        timestamp = data_read[Enum.timestamp]
        fhr = data_read[Enum.fhr]
        uc = data_read[Enum.uc]

        #print data_read[0,0], self._data_write[0,0]
        self.assertEqual(timestamp[0,0], t1[0,0])
        self.assertEqual(timestamp[1,0], t1[1,0])

        self.assertEqual(fhr[0], t2[0,0])
        self.assertEqual(fhr[1], t2[1,0])

        self.assertEqual(uc[0], t3[0,0])
        self.assertEqual(uc[1], t3[1,0])

    def test_read_matlab(self):

        file1 = os.path.join('unittest_files','1001.mat')
        aData,dummy = self._dataLoader.readMatlabFile(file1)

        timestamp = aData[Enum.timestamp]
        fhr = aData[Enum.fhr]
        uc = aData[Enum.uc]

        self.assertEqual(timestamp[0], 1)
        self.assertEqual(timestamp[1], 2)

        self.assertEqual(fhr[0], 150.5)
        self.assertEqual(fhr[3], 151.25)
        self.assertEqual(fhr[18978], 94.75)

        self.assertEqual(uc[0], 7.0)
        self.assertEqual(uc[1], 8.5)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()